// Library:       MQSeries
// Component:     C++ sample Programs
// Part:          IMQDPUT.CPP
//
// Description:   Sample C++ program that puts messages to a distribution
//                list containing two queues.
//
//                IMQDUT has 3 parameters:
//                - the name of a queue (required)
//                - the name of another queue (required)
//                - the name of a queue manager (optional)
//
//                If no queue manager name is given, the queue manager
//                defaults to the default local queue manager.
//
//  Statement:     Licensed Materials - Property of IBM
//
//                 04L1773, 5765-B73
//                 04L1802, 5639-B42
//                 04L1788, 5765-B74
//                 04L1816, 5765-B75
//                 04L1830, 5639-B43
//                 (C) Copyright IBM Corp. 1994, 1998
//
//                U.S. Government Users Restricted Rights - use,
//                duplication or disclosure restricted by GSA ADP
//                Schedule Contract with IBM Corp.

extern "C" {
#include <stdio.h>
#include <stdlib.h>
}

#include <imqi.hpp> // MQSeries MQI

int main ( int argc, char * * argv ) {
  FILE * fp ;                      /* Console                       */

  ImqQueueManager mgr;             /* Queue manager                 */
  ImqQueue queueA;                 /* Queue                         */
  ImqQueue queueB;                 /* Another queue                 */
  ImqMessage msg;                  /* Data message                  */
  int      buflen;                 /* buffer length                 */
  char     buffer[100];            /* message buffer                */

  ImqDistributionList dlist;       /* Distribution list.            */

  printf( "Sample IMQDPUT start\n" );
  if ( argc < 3 ) {
    printf( "Usage : %s queue1 queue2 (queue manager)\n", argv[0] );
    printf( "Missing parameter.\n" );
    return ( 99 );
  }

  /******************************************************************/
  /*                                                                */
  /*   Connect to queue manager                                     */
  /*                                                                */
  /******************************************************************/
  if ( argc > 2 ) {
    mgr.setName( argv[3] );
  }

  if ( ! mgr.connect( ) ) {

    /* stop if it failed */
    printf( "ImqQueueManager::connect ended with reason code %ld\n",
            (long)mgr.reasonCode( ) );
    exit ( (int)mgr.reasonCode( ) );
  }

  /******************************************************************/
  /*                                                                */
  /*   Connection successful. Obtain queue manager's name.          */
  /*                                                                */
  /******************************************************************/
  ImqString strManagerName( mgr.name( ) );
  printf("Successful connection to queue manager %s\n",
                                        (char *)strManagerName );

  /******************************************************************/
  /*                                                                */
  /*   Associate distribution list with queue manager.              */
  /*                                                                */
  /******************************************************************/
  dlist.setConnectionReference( mgr );

  /******************************************************************/
  /*                                                                */
  /*   Associate queues with queue manager.                         */
  /*                                                                */
  /******************************************************************/
  queueA.setConnectionReference( mgr );
  queueB.setConnectionReference( mgr );

  /******************************************************************/
  /*                                                                */
  /*   Use parameters to name target queues.                        */
  /*                                                                */
  /******************************************************************/
  queueA.setName( argv[ 1 ] );
  queueB.setName( argv[ 2 ] );
  printf( "First target queue is %s\n", (char *)queueA.name( ) );
  printf( "Second target queue is %s\n", (char *)queueB.name( ) );

  queueA.setQueueManagerName( (char *)strManagerName );
  queueB.setQueueManagerName( (char *)strManagerName );
  printf( "First target queue has queue manager %s\n",
          (char *)queueA.queueManagerName( ) );
  printf( "Second target queue has queue manager %s\n",
          (char *)queueB.queueManagerName( ) );

  /******************************************************************/
  /*                                                                */
  /*   Associate the queues with the distribution list.             */
  /*                                                                */
  /******************************************************************/
  queueA.setDistributionListReference( dlist );
  queueB.setDistributionListReference( dlist );

  /******************************************************************/
  /*                                                                */
  /*   Open the distribution list for output.                       */
  /*                                                                */
  /******************************************************************/
  dlist.setOpenOptions( MQOO_OUTPUT /* open queue for output        */
    + MQOO_FAIL_IF_QUIESCING );     /* but not if MQM stopping      */
  dlist.open( );

  /* report reason, if any; stop if failed      */
  if ( dlist.reasonCode( ) ) {
    printf( "ImqDst::open ended with reason code %ld\n",
            (long)dlist.reasonCode( ) );

    printf( "First target queue has reason code %ld\n",
            (long)queueA.reasonCode( ) );
    printf( "Second target queue has reason code %ld\n",
            (long)queueB.reasonCode( ) );
  }

  if ( dlist.completionCode( ) == MQCC_FAILED ) {
    printf( "unable to open distributon list for output\n" );
  }

  /******************************************************************/
  /*                                                                */
  /*   Read lines from the stdin and put them to the distribution   */
  /*   list. Repeat until null line entered or failure occurs.      */
  /*                                                                */
  /******************************************************************/
  fp = stdin;
  msg.useEmptyBuffer( buffer, sizeof( buffer ) );
  msg.setFormat( MQFMT_STRING );      /* character string format    */

  while ( dlist.completionCode( ) != MQCC_FAILED ) {

    if ( fgets( buffer, (int)sizeof(buffer) - 1, fp ) ) {
      buflen = (int)strlen( buffer ) - 1 ; /* length w/o line-end   */
      buffer[ buflen ] = '\0' ;            /* add line-end          */
    } else {
      buflen = 0;           /* treat EOF same as null line          */
    }

    /****************************************************************/
    /*                                                              */
    /*   Put each buffer to the distribution list.                  */
    /*                                                              */
    /****************************************************************/
    if ( buflen > 0 ) {
      msg.setMessageLength( buflen );
      if ( ! dlist.put( msg ) ) {

        /* report reason, if any */
        printf( "ImqDistributionList::put ended with reason code %ld\n",
                (long)dlist.reasonCode( ) );
      }

    } else {
      /* quit loop when empty line is read */
      break ;
    }
  }

  /******************************************************************/
  /*                                                                */
  /*   Close the target distribution list (if it was opened).       */
  /*                                                                */
  /******************************************************************/
  if ( ! dlist.close( ) ) {

    /* report reason, if any     */
    printf( "ImqDistributionList::close ended with reason code %ld\n",
            (long)dlist.reasonCode( ) );
  }

  /******************************************************************/
  /*                                                                */
  /*   Disconnect from MQM if not already connected (the            */
  /*   ImqQueueManager object handles this situation automatically) */
  /*                                                                */
  /******************************************************************/
  if ( ! mgr.disconnect( ) ) {

    /* report reason, if any     */
    printf( "ImqQueueManager::disconnect ended with reason code %ld\n",
            (long)mgr.reasonCode( ) );
  }

  /******************************************************************/
  /*                                                                */
  /* END OF IMQDPUT                                                 */
  /*                                                                */
  /******************************************************************/
  printf( "Sample IMQDPUT end\n" );
  return( 0 );
}
